import { JsAstEditor, SvelteAstEditor, HtmlAstEditor, CssAstEditor } from '@svelte-add/ast-manipulation';
import { Colors } from 'picocolors/types.js';
import { ChildProcess } from 'child_process';
export { default as dedent } from 'dedent';

type Primitive = Readonly<string | boolean | number>;
type PromptOption<Value> = Value extends Primitive ? {
    value: Value;
    label?: string;
    hint?: string;
} : {
    value: Value;
    label: string;
    hint?: string;
};

type PrettierData = {
    installed: boolean;
};
type TypescriptData = {
    installed: boolean;
};
type SvelteKitData = {
    installed: boolean;
    libDirectory: string;
    routesDirectory: string;
};
type Workspace<Args extends OptionDefinition> = {
    options: OptionValues<Args>;
    cwd: string;
    prettier: PrettierData;
    typescript: TypescriptData;
    kit: SvelteKitData;
};

type BooleanQuestion = {
    type: "boolean";
    default: boolean;
};
type StringQuestion = {
    type: "string";
    default: string;
};
type NumberQuestion = {
    type: "number";
    default: number;
};
type SelectQuestion<Value = any> = {
    type: "select";
    default: Value;
    options: PromptOption<Value>[];
};
type BaseQuestion = {
    question: string;
    condition?: (options: OptionValues<any>) => boolean;
};
type Question = BaseQuestion & (BooleanQuestion | StringQuestion | NumberQuestion | SelectQuestion);
type OptionDefinition = Record<string, Question>;
type OptionValues<Args extends OptionDefinition> = {
    [K in keyof Args]: Args[K] extends StringQuestion ? string : Args[K] extends BooleanQuestion ? boolean : Args[K] extends NumberQuestion ? number : Args[K] extends SelectQuestion<infer Value> ? Value : never;
};

type BaseFile<Args extends OptionDefinition> = {
    name: (options: Workspace<Args>) => string;
    condition?: ConditionDefinition<Args>;
};
type ScriptFileEditorArgs<Args extends OptionDefinition> = JsAstEditor & Workspace<Args>;
type ScriptFileType<Args extends OptionDefinition> = {
    contentType: "script";
    content: (editor: ScriptFileEditorArgs<Args>) => void;
};
type ScriptFile<Args extends OptionDefinition> = ScriptFileType<Args> & BaseFile<Args>;
type TextFileEditorArgs<Args extends OptionDefinition> = {
    content: string;
} & Workspace<Args>;
type TextFileType<Args extends OptionDefinition> = {
    contentType: "text";
    content: (editor: TextFileEditorArgs<Args>) => string;
};
type TextFile<Args extends OptionDefinition> = TextFileType<Args> & BaseFile<Args>;
type SvelteFileEditorArgs<Args extends OptionDefinition> = SvelteAstEditor & Workspace<Args>;
type SvelteFileType<Args extends OptionDefinition> = {
    contentType: "svelte";
    content: (editor: SvelteFileEditorArgs<Args>) => void;
};
type SvelteFile<Args extends OptionDefinition> = SvelteFileType<Args> & BaseFile<Args>;
type JsonFileEditorArgs<Args extends OptionDefinition> = {
    data: any;
} & Workspace<Args>;
type JsonFileType<Args extends OptionDefinition> = {
    contentType: "json";
    content: (editor: JsonFileEditorArgs<Args>) => void;
};
type JsonFile<Args extends OptionDefinition> = JsonFileType<Args> & BaseFile<Args>;
type HtmlFileEditorArgs<Args extends OptionDefinition> = HtmlAstEditor & Workspace<Args>;
type HtmlFileType<Args extends OptionDefinition> = {
    contentType: "html";
    content: (editor: HtmlFileEditorArgs<Args>) => void;
};
type HtmlFile<Args extends OptionDefinition> = HtmlFileType<Args> & BaseFile<Args>;
type CssFileEditorArgs<Args extends OptionDefinition> = CssAstEditor & Workspace<Args>;
type CssFileType<Args extends OptionDefinition> = {
    contentType: "css";
    content: (editor: CssFileEditorArgs<Args>) => void;
};
type CssFile<Args extends OptionDefinition> = CssFileType<Args> & BaseFile<Args>;
type FileTypes<Args extends OptionDefinition> = ScriptFile<Args> | TextFile<Args> | SvelteFile<Args> | JsonFile<Args> | HtmlFile<Args> | CssFile<Args>;

type PreconditionParameters<Args extends OptionDefinition> = {
    workspace: Workspace<Args>;
    fileExists: (path: string) => Promise<void>;
    fileContains: (path: string, expectedContent: string) => Promise<void>;
};
type Postcondition<Args extends OptionDefinition> = {
    name: string;
    run: (params: PreconditionParameters<Args>) => Promise<void>;
};

type ConditionDefinition<Args extends OptionDefinition> = (Workspace: Workspace<Args>) => boolean;
type WebsiteMetadata = {
    logo: string;
    keywords: string[];
    documentation: string;
};
type AdderConfigEnvironments = {
    svelte: boolean;
    kit: boolean;
};
type AdderConfigMetadata = {
    id: string;
    name: string;
    description: string;
    environments: AdderConfigEnvironments;
    website?: WebsiteMetadata;
};
type PackageDefinition<Args extends OptionDefinition> = {
    name: string;
    version: string;
    dev: boolean;
    condition?: ConditionDefinition<Args>;
};
type BaseAdderConfig<Args extends OptionDefinition> = {
    metadata: AdderConfigMetadata;
    options: Args;
    runsAfter?: string[];
    integrationType: string;
};
type InlineAdderConfig<Args extends OptionDefinition> = BaseAdderConfig<Args> & {
    integrationType: "inline";
    packages: PackageDefinition<Args>[];
    files: FileTypes<Args>[];
    nextSteps?: (data: {
        options: OptionValues<Args>;
        cwd: string;
        colors: Colors;
        docs: string | undefined;
    }) => string[];
    installHook?: (workspace: Workspace<Args>) => Promise<void>;
    uninstallHook?: (workspace: Workspace<Args>) => Promise<void>;
};
type ExternalAdderConfig<Args extends OptionDefinition> = BaseAdderConfig<Args> & {
    integrationType: "external";
    command: string;
    environment?: Record<string, string>;
};
type AdderConfig<Args extends OptionDefinition> = InlineAdderConfig<Args> | ExternalAdderConfig<Args>;
declare function defineAdderConfig<Args extends OptionDefinition>(config: AdderConfig<Args>): AdderConfig<Args>;
type Adder<Args extends OptionDefinition> = {
    config: AdderConfig<Args>;
    checks: AdderCheckConfig<Args>;
    tests?: AdderTestConfig<Args>;
};
declare function defineAdder<Args extends OptionDefinition>(config: AdderConfig<Args>, checks: AdderCheckConfig<Args>, tests?: AdderTestConfig<Args>): Adder<Args>;
type Tests = {
    expectProperty: (selector: string, property: string, expectedValue: string) => Promise<void>;
    elementExists: (selector: string) => Promise<void>;
    click: (selector: string, path?: string) => Promise<void>;
    expectUrlPath: (path: string) => void;
};
type TestDefinition<Args extends OptionDefinition> = {
    name: string;
    run: (tests: Tests) => Promise<void>;
    condition?: (options: OptionValues<Args>) => boolean;
};
type AdderTestConfig<Args extends OptionDefinition> = {
    files: FileTypes<Args>[];
    options: Args;
    optionValues: OptionValues<Args>[];
    runSynchronously?: boolean;
    command?: string;
    tests: TestDefinition<Args>[];
};
declare function defineAdderTests<Args extends OptionDefinition>(tests: AdderTestConfig<Args>): AdderTestConfig<Args>;
declare function defineAdderOptions<const Args extends OptionDefinition>(options: Args): Args;
type MaybePromise<T> = Promise<T> | T;
type Precondition = {
    name: string;
    run: () => MaybePromise<{
        success: boolean;
        message: string | undefined;
    }>;
};
type AdderCheckConfig<Args extends OptionDefinition> = {
    options: Args;
    preconditions?: Precondition[];
    postconditions?: Postcondition<Args>[];
};
declare function defineAdderChecks<Args extends OptionDefinition>(checks: AdderCheckConfig<Args>): AdderCheckConfig<Args>;

declare function executeCli(command: string, commandArgs: string[], cwd: string, options?: {
    onData?: (data: string, program: ChildProcess, resolve: (value?: any) => any) => void;
    stdio?: "pipe" | "inherit";
    env?: Record<string, string>;
}): Promise<any>;

export { defineAdder, defineAdderChecks, defineAdderConfig, defineAdderOptions, defineAdderTests, executeCli };
